package dao.database_vero;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.Statement;

import dao.PostDAO;
import dao.PostDTO;

public class HsqldbPostDAO 
implements PostDAO {

	// === Costanti letterali per non sbagliarsi a scrivere !!! ============================
	
	static final String post_table = "post";

	// -------------------------------------------------------------------------------------

	static final String post_id = "ID";
	static final String post_giorno = "Giorno";
	static final String post_ora = "Ora";
	static final String post_testo = "Testo";
	
	// == STATEMENT SQL ====================================================================

	// INSERT INTO usertable ( namecolumn,surnamecolumn, ...) VALUES ( ?,?, ... );
	static final String insert = 
		"INSERT " +
			"INTO " + post_table + " ( " + 
				post_id + ", " + post_giorno +", " + post_ora + ", " + post_testo + 
			") " +
			"VALUES (?,?,?,?) ;"
		;
	
	// SELECT * FROM usertable WHERE idcolumn = ?;
	static String read_by_id = 
		"SELECT * " +
			"FROM " + post_table + " " +
			"WHERE " + post_id + " = ? ;"
		;

	// -------------------------------------------------------------------------------------

	// CREATE usertable ( idcolumn INT NOT NULL IDENTITY, namecolumn VARCHAR(50), ..., UNIQUE ( idcolumn ) );
	static String create = 
		"CREATE " +
			"TABLE " + post_table +" ( " +
				post_id + " INT NOT NULL, " +
				post_giorno + " VARCHAR(20), " +
				post_ora + " VARCHAR(10), " +
				post_testo + " VARCHAR(250) " +
			") ;"
		;
	static String drop = 
		"DROP " +
			"TABLE " + post_table + " ;"
		;
	
	// === METODI DAO =========================================================================

	/**
	 * C
	 */
	@Override
	public int createPost(PostDTO t) {
		// --- 1. Dichiarazione della variabile per il risultato ---
		int result = -1;
		// --- 2. Controlli preliminari sui dati in ingresso ---
		if ( t == null )  {
			System.err.println(this.getClass().getName()+".create(): failed to insert a null");
			return result;
		}
		// --- 3. Apertura della connessione ---
		Connection conn = HsqldbDAOFactory.createConnection();
		// --- 4. Tentativo di accesso al db e impostazione del risultato ---
		try {
			// --- a. Crea (se senza parametri) o prepara (se con parametri) lo statement
			PreparedStatement prep_stmt = conn.prepareStatement(insert);
			// --- b. Pulisci e imposta i parametri (se ve ne sono)
			prep_stmt.clearParameters();
			prep_stmt.setInt(1, t.getId());
			prep_stmt.setString(2, t.getGiorno());
			prep_stmt.setString(3, t.getOra());
			prep_stmt.setString(4, t.getTesto());
			// --- c. Esegui l'azione sul database ed estrai il risultato (se atteso)
			prep_stmt.executeUpdate();
			// --- d. Cicla sul risultato (se presente) pe accedere ai valori di ogni sua tupla e setta i risultati
			result = t.getId();
			// --- e. Rilascia la struttura dati del risultato
			// n.d.
			// --- f. Rilascia la struttura dati dello statement
			prep_stmt.close();
		}
		// --- 5. Gestione di eventuali eccezioni ---
		catch (Exception e) {
			System.err.println(this.getClass().getName()+".create(): failed to insert\n"+e);
			e.printStackTrace();
			result = -2;
		}
		// --- 6. Rilascio, SEMPRE E COMUNQUE, la connessione prima di restituire il controllo al chiamante
		finally {
			HsqldbDAOFactory.closeConnection(conn);
		}
		// --- 7. Restituzione del risultato (eventualmente di fallimento)
		return result;
	}

	@Override
	public PostDTO readPost(int id) {
		// --- 1. Dichiarazione della variabile per il risultato ---
		PostDTO result = null;
		// --- 2. Controlli preliminari sui dati in ingresso ---
		if ( id < 0 )  {
			System.err.println(this.getClass().getName()+".read(): failed to read with negative id");
			return result;
		}
		// --- 3. Apertura della connessione ---
		Connection conn = HsqldbDAOFactory.createConnection();
		// --- 4. Tentativo di accesso al db e impostazione del risultato ---
		try {
			// --- a. Crea (se senza parametri) o prepara (se con parametri) lo statement
			PreparedStatement prep_stmt = conn.prepareStatement(read_by_id);
			// --- b. Pulisci e imposta i parametri (se ve ne sono)
			prep_stmt.clearParameters();
			prep_stmt.setInt(1, id);
			// --- c. Esegui l'azione sul database ed estrai il risultato (se atteso)
			ResultSet rs = prep_stmt.executeQuery();
			// --- d. Cicla sul risultato (se presente) pe accedere ai valori di ogni sua tupla
			if ( rs.next() ) {
				PostDTO post = new PostDTO();
				post.setId(rs.getInt(post_id));
				post.setGiorno(rs.getString(post_giorno));
				post.setOra(rs.getString(post_ora));
				post.setTesto(rs.getString(post_testo));
				result = post;
			}
			// --- e. Rilascia la struttura dati del risultato
			rs.close();
			// --- f. Rilascia la struttura dati dello statement
			prep_stmt.close();
		}
		// --- 5. Gestione di eventuali eccezioni ---
		catch (Exception e) {
			System.err.println(this.getClass().getName()+".read(): failed to retrieve by id = " + id+"\n"+e);
			e.printStackTrace();
		}
		// --- 6. Rilascio, SEMPRE E COMUNQUE, la connessione prima di restituire il controllo al chiamante
		finally {
			HsqldbDAOFactory.closeConnection(conn);
		}
		// --- 7. Restituzione del risultato (eventualmente di fallimento)
		return result;
	}

	// --------------------------------------------------
	
	@Override
	public boolean createTable() {
		// --- 1. Dichiarazione della variabile per il risultato ---
		boolean result = false;
		// --- 2. Controlli preliminari sui dati in ingresso ---
		// n.d.
		// --- 3. Apertura della connessione ---
		Connection conn = HsqldbDAOFactory.createConnection();
		// --- 4. Tentativo di accesso al db e impostazione del risultato ---
		try {
			// --- a. Crea (se senza parametri) o prepara (se con parametri) lo statement
			Statement stmt = conn.createStatement();
			// --- b. Pulisci e imposta i parametri (se ve ne sono)
			// n.d.
			// --- c. Esegui l'azione sul database ed estrai il risultato (se atteso)
			stmt.execute(create);
			// --- d. Cicla sul risultato (se presente) pe accedere ai valori di ogni sua tupla
			// n.d. Qui devo solo dire al chiamante che è andato tutto liscio
			result = true;
			// --- e. Rilascia la struttura dati del risultato
			// n.d.
			// --- f. Rilascia la struttura dati dello statement
			stmt.close();
		}
		// --- 5. Gestione di eventuali eccezioni ---
		catch (Exception e) {
			System.err.println(this.getClass().getName()+".createTable(): failed to create table\n"+e);
		}
		// --- 6. Rilascio, SEMPRE E COMUNQUE, la connessione prima di restituire il controllo al chiamante
		finally {
			HsqldbDAOFactory.closeConnection(conn);
		}
		// --- 7. Restituzione del risultato (eventualmente di fallimento)
		return result;
	}

	@Override
	public boolean dropTable() {
		// --- 1. Dichiarazione della variabile per il risultato ---
		boolean result = false;
		// --- 2. Controlli preliminari sui dati in ingresso ---
		// n.d.
		// --- 3. Apertura della connessione ---
		Connection conn = HsqldbDAOFactory.createConnection();
		// --- 4. Tentativo di accesso al db e impostazione del risultato ---
		try {
			// --- a. Crea (se senza parametri) o prepara (se con parametri) lo statement
			Statement stmt = conn.createStatement();
			// --- b. Pulisci e imposta i parametri (se ve ne sono)
			// n.d.
			// --- c. Esegui l'azione sul database ed estrai il risultato (se atteso)
			stmt.execute(drop);
			// --- d. Cicla sul risultato (se presente) pe accedere ai valori di ogni sua tupla
			// n.d. Qui devo solo dire al chiamante che è andato tutto a posto.
			result = true;
			// --- e. Rilascia la struttura dati del risultato
			// n.d.
			// --- f. Rilascia la struttura dati dello statement
			stmt.close();
		}
		// --- 5. Gestione di eventuali eccezioni ---
		catch (Exception e) {
			System.err.println(this.getClass().getName()+".dropTable(): failed to drop table\n"+e);
		}
		// --- 6. Rilascio, SEMPRE E COMUNQUE, la connessione prima di restituire il controllo al chiamante
		finally {
			HsqldbDAOFactory.closeConnection(conn);
		}
		// --- 7. Restituzione del risultato (eventualmente di fallimento)
		return result;
	}

}
